/*
 * This file is part of Barista.
 * Copyright (C) 2007-2010 Xavier Clerc.
 *
 * Barista is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Barista is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.io.FileOutputStream;
import java.util.Collections;
import java.util.List;
import static java.util.Arrays.asList;

import fr.x9c.barista.api.*;
import static fr.x9c.barista.api.Instruction.*;
import static tests.Common.*;

public final class Source {

    private static final FieldRef systOut =
	new FieldRef("java.lang.System",
		     "out",
		     new Descriptor.Class("java.io.PrintStream"));

    public static void main(final String[] args) throws Throwable {
	final List<Instruction> instructions =
	    asList(/* d2f */
		   new GETSTATIC(systOut),
		   DCONST_1,
		   D2F,
		   new INVOKEVIRTUAL(new MethodRef("java.io.PrintStream",
						   "println",
						   Descriptor.VOID,
						   descs(Descriptor.FLOAT))),

		   /* d2i */
		   new GETSTATIC(systOut),
		   DCONST_1,
		   D2I,
		   new INVOKEVIRTUAL(new MethodRef("java.io.PrintStream",
						   "println",
						   Descriptor.VOID,
						   descs(Descriptor.INT))),

		   /* d2l */
		   new GETSTATIC(systOut),
		   DCONST_1,
		   D2L,
		   new INVOKEVIRTUAL(new MethodRef("java.io.PrintStream",
						   "println",
						   Descriptor.VOID,
						   descs(Descriptor.LONG))),

		   /* f2d */
		   new GETSTATIC(systOut),
		   FCONST_0,
		   F2D,
		   new INVOKEVIRTUAL(new MethodRef("java.io.PrintStream",
						   "println",
						   Descriptor.VOID,
						   descs(Descriptor.DOUBLE))),

		   /* f2i */
		   new GETSTATIC(systOut),
		   FCONST_0,
		   F2I,
		   new INVOKEVIRTUAL(new MethodRef("java.io.PrintStream",
						   "println",
						   Descriptor.VOID,
						   descs(Descriptor.INT))),

		   /* f2l */
		   new GETSTATIC(systOut),
		   FCONST_0,
		   F2L,
		   new INVOKEVIRTUAL(new MethodRef("java.io.PrintStream",
						   "println",
						   Descriptor.VOID,
						   descs(Descriptor.LONG))),

		   /* i2b */
		   ICONST_5,
		   I2B,
		   new GETSTATIC(systOut),
		   SWAP,
		   new INVOKEVIRTUAL(new MethodRef("java.io.PrintStream",
						   "println",
						   Descriptor.VOID,
						   descs(Descriptor.INT))),

		   /* i2c */
		   ICONST_5,
		   I2C,
		   new GETSTATIC(systOut),
		   SWAP,
		   new INVOKEVIRTUAL(new MethodRef("java.io.PrintStream",
						   "println",
						   Descriptor.VOID,
						   descs(Descriptor.INT))),

		   /* i2d */
		   new GETSTATIC(systOut),
		   ICONST_5,
		   I2D,
		   new INVOKEVIRTUAL(new MethodRef("java.io.PrintStream",
						   "println",
						   Descriptor.VOID,
						   descs(Descriptor.DOUBLE))),

		   /* i2f */
		   new GETSTATIC(systOut),
		   ICONST_5,
		   I2F,
		   new INVOKEVIRTUAL(new MethodRef("java.io.PrintStream",
						   "println",
						   Descriptor.VOID,
						   descs(Descriptor.FLOAT))),

		   /* i2l */
		   new GETSTATIC(systOut),
		   ICONST_5,
		   I2L,
		   new INVOKEVIRTUAL(new MethodRef("java.io.PrintStream",
						   "println",
						   Descriptor.VOID,
						   descs(Descriptor.LONG))),

		   /* i2s */
		   ICONST_5,
		   I2S,
		   new GETSTATIC(systOut),
		   SWAP,
		   new INVOKEVIRTUAL(new MethodRef("java.io.PrintStream",
						   "println",
						   Descriptor.VOID,
						   descs(Descriptor.INT))),

		   /* l2d */
		   new GETSTATIC(systOut),
		   LCONST_1,
		   L2D,
		   new INVOKEVIRTUAL(new MethodRef("java.io.PrintStream",
						   "println",
						   Descriptor.VOID,
						   descs(Descriptor.DOUBLE))),

		   /* l2f */
		   new GETSTATIC(systOut),
		   LCONST_1,
		   L2F,
		   new INVOKEVIRTUAL(new MethodRef("java.io.PrintStream",
						   "println",
						   Descriptor.VOID,
						   descs(Descriptor.FLOAT))),

		   /* l2i */
		   new GETSTATIC(systOut),
		   LCONST_1,
		   L2I,
		   new INVOKEVIRTUAL(new MethodRef("java.io.PrintStream",
						   "println",
						   Descriptor.VOID,
						   descs(Descriptor.INT))),

		   RETURN);
	final Method main = compileMethod(instructions);
	final ClassDefinition cd = compileClass(null,
						null,
						null,
						null,
						null,
						null,
						asList(main));
	final byte[] bc = ByteCode.encode(cd);
	FileOutputStream out = new FileOutputStream("pack/Test.class");
	out.write(bc);
	out.flush();
    }

}
